//----------------------------------------------------------------------------
// ObjectWindows
// Copyright (c) 1996, 1996 by Borland International, All Rights Reserved
//
// $Revision:   10.3  $
//
// Docking window example main source file
//----------------------------------------------------------------------------
#include "owlpch.h"
#include <owl/printer.h>
#include <owl/buttonga.h>
#include <owl/textgadg.h>
#include <owl/controlb.h>
#include <owl/controlg.h>
#include <owl/statusba.h>
#include <owl/docking.h>
#include <owl/combobox.h>
#include <owl/edit.h>
#include <owl/decmdifr.h>
#include <owl/editfile.h>
#include <owl/uihelper.h>
#include <owl/uimetric.h>
//#include <dir.h>
#include "menubar.h"

////////////////////////////////////////////////////////////////////////////////
//
class TMenuBarProxy {
	public:
  	virtual TGadgetWindow::TTileDirection GetDirection() const
              { return TGadgetWindow::Horizontal;}
  	virtual TFont& GetFont() = 0;
  	virtual TWindow* GetTarget() = 0;
};
////////////////////////////////////////////////////////////////////////////////
//
//
class TMenuBarItem {
	protected:
  	TMenuBarItem(TMenuBarProxy* proxy, int id):Proxy(proxy),Id(id){}
	public:
  	virtual ~TMenuBarItem(){}

    virtual void Paint(TDC& dc, TRect& rect)=0;
    virtual void GetSize(TSize& size) = 0;
    int	 GetId() { return Id;}
		virtual void LButtonDown(uint modKeys, TPoint& p, TRect& rect){}

	protected_data:
  	TMenuBarProxy*	Proxy;
    int						 	Id;
};
////////////////////////////////////////////////////////////////////////////////
//
// Simple Text
class TTextMenuBarItem : public TMenuBarItem {
	public:
  	enum TTextStyle{msLeftText,msCenterText,msRightText,};
	public:
  	TTextMenuBarItem(TMenuBarProxy* proxy, int id, LPCTSTR text,
                     TTextStyle stype = msLeftText);
  	virtual ~TTextMenuBarItem();
    void SetFont(TFont* font);

		virtual void GetSize(TSize& size);
    virtual void Paint(TDC& dc, TRect& rect);
    
	protected:
  	LPTSTR 	Text;
  	int 		TextLen;
    uint    Style;
    TFont*  Font;
};
//
TTextMenuBarItem::TTextMenuBarItem(TMenuBarProxy* proxy, int id,
                                   LPCTSTR text, TTextStyle style)
:
	TMenuBarItem(proxy, id),
	Style(style),
  Font(0),
	Text(strnewdup(text))
{
	TextLen = _tcslen(Text);
}
//
TTextMenuBarItem::~TTextMenuBarItem()
{
	delete[] Text;
  delete Font;
}
//
void TTextMenuBarItem::SetFont(TFont* font)
{
	delete Font;
  Font = font;
}
//
void TTextMenuBarItem::Paint(TDC& dc, TRect& rect)
{
  if (!Font)
 	  dc.SelectObject(Proxy->GetFont());
 	else
   	dc.SelectObject(*Font);
  TColor textColor = TColor::SysMenuText;//GetEnabledColor();
  //if(!GetEnabled())
  //	textColor = TSystem::Has3dUI() ? TColor::Sys3dHilight : TColor::SysGrayText;
 	TColor oldTxColor  = dc.SetTextColor(textColor);
  TColor oldBkColor  = dc.SetBkColor(TColor::SysMenu);
	if(Text){
    uint16 i_fmt[] = {DT_LEFT,DT_CENTER,DT_RIGHT};
  	uint16 format = uint16(DT_EXPANDTABS|i_fmt[Style]);
		dc.DrawText(Text, TextLen, rect, format);
  }
  else{
	  TColor color = dc.SetBkColor(TColor::Sys3dFace);
  	dc.ExtTextOut(0,0, ETO_OPAQUE, &rect, 0, 0);
	  dc.SetBkColor(color);
  }
  dc.SetTextColor(oldTxColor);
  dc.SetBkColor(oldBkColor);
  dc.RestoreFont();
}
//
void TTextMenuBarItem::GetSize(TSize& size)
{
  TFont* font = Font;
  if (font == 0)
    font = &(Proxy->GetFont());

  if (!font)
    return;
  size.cx += font->GetTextExtent(Text).cx;
  size.cy += font->GetHeight() + 2;
}
////////////////////////////////////////////////////////////////////////////////
//
//
class TPopupMenuBarItem : public TTextMenuBarItem {
	public:
  	TPopupMenuBarItem(TMenuBarProxy* proxy, int id, HMENU menu,
                      LPCTSTR text, TTextStyle stype = msLeftText);
  	virtual ~TPopupMenuBarItem();
		virtual void GetSize(TSize& size);
    virtual void Paint(TDC& dc, TRect& rect);
		virtual void LButtonDown(uint modKeys, TPoint& p, TRect& rect);
	protected:
  	HMENU	hMenu;
};
//
TPopupMenuBarItem::TPopupMenuBarItem(TMenuBarProxy* proxy, int id, HMENU menu,
                                     LPCTSTR text, TTextStyle stype)
:
	TTextMenuBarItem(proxy, id, text, stype),
  hMenu(menu)
{
}
TPopupMenuBarItem::~TPopupMenuBarItem()
{
	::DestroyMenu(hMenu);
}
//
void TPopupMenuBarItem::GetSize(TSize& size)
{
	TTextMenuBarItem::GetSize(size);
  size.cx += 10;
}
//
void TPopupMenuBarItem::Paint(TDC& dc, TRect& rect)
{
	TTextMenuBarItem::Paint(dc, rect);
}
//
void TPopupMenuBarItem::LButtonDown(uint modKeys, TPoint&, TRect& rect)
{
  TPoint p2(rect.BottomLeft());
  Proxy->GetTarget()->ClientToScreen(p2);
  TPopupMenu PopupMenu(hMenu);
  PopupMenu.TrackPopupMenu(TPM_LEFTALIGN|TPM_LEFTBUTTON, p2, 0, *Proxy->GetTarget());
}
////////////////////////////////////////////////////////////////////////////////
//
// Separator
class TSeparatorMenuBarItem : public TMenuBarItem {
	public:
  	TSeparatorMenuBarItem(TMenuBarProxy* proxy)
    	: TMenuBarItem(proxy, 0){}
  	virtual ~TSeparatorMenuBarItem(){}
		virtual void GetSize(TSize& size);
    virtual void Paint(TDC& dc, TRect& rect);

  private:
    // Hidden to prevent accidental copying or assignment
    //
    TSeparatorMenuBarItem(const TSeparatorMenuBarItem&);
    TSeparatorMenuBarItem& operator =(const TSeparatorMenuBarItem&);
};
//
void TSeparatorMenuBarItem::GetSize(TSize& size)
{
	size.cx += 2;
	size.cy += 2;
}
//
void TSeparatorMenuBarItem::Paint(TDC& dc, TRect& rect)
{
	if(Proxy->GetDirection() == TGadgetWindow::Vertical){
		TRect boundsRect(rect);
  	boundsRect.top	 += 2;
    boundsRect.bottom = boundsRect.top+3;
    TUIBorder::DrawEdge(dc, boundsRect, TUIBorder::RaisedInner, TUIBorder::Rect);
  }
  else{
		TRect boundsRect(rect);
  	boundsRect.left	 += 2;
  	boundsRect.right = boundsRect.left+3;
		TUIBorder::DrawEdge(dc, boundsRect, TUIBorder::RaisedInner, TUIBorder::Rect);
  }
}
////////////////////////////////////////////////////////////////////////////////
//
class TMenuBarGadget : public TGadget{
	public:
  	TMenuBarGadget(TMenuBarItem* item);
    ~TMenuBarGadget();

    // Override from TGadget
    //
    virtual void LButtonDown(uint modKeys, TPoint& p);
    virtual void Paint(TDC& dc);
		//virtual TColor 	GetEnabledColor() const;
    virtual void PaintBorder(TDC& dc);
    virtual void GetDesiredSize(TSize &size);
    virtual void Invalidate();

  protected_data:
		TMenuBarItem* MenuItem;

  private:
    // Hidden to prevent accidental copying or assignment
    //
    TMenuBarGadget(const TMenuBarGadget&);
    TMenuBarGadget& operator =(const TMenuBarGadget&);
};
//
TMenuBarGadget::TMenuBarGadget(TMenuBarItem* item)
:
	TGadget(item->GetId()),
	MenuItem(item)
{
  Margins.Left = Margins.Right = TUIMetric::CxFixedFrame;
  SetShrinkWrap(false, true);
}
//
TMenuBarGadget::~TMenuBarGadget()
{
}
//
void TMenuBarGadget::Paint(TDC& dc)
{
  PaintBorder(dc);

  TRect  innerRect;
  GetInnerRect(innerRect);
	MenuItem->Paint(dc,innerRect);
}
//
void TMenuBarGadget::GetDesiredSize(TSize& size)
{
	// for flat style set borders minimum 1
	if((Window->GetFlatStyle()&TGadgetWindow::FlatStandard) &&
      GetBorderStyle()==None && Borders.Left == 0 && Borders.Top == 0 &&
      Borders.Right==0 && Borders.Bottom==0)
  	Borders.Left = Borders.Top = Borders.Right = Borders.Bottom = 1;

  TGadget::GetDesiredSize(size);
	MenuItem->GetSize(size);
}
//
void TMenuBarGadget::Invalidate()
{
  TRect  innerRect;

  GetInnerRect(innerRect);
  InvalidateRect(innerRect, false);
}
//
void TMenuBarGadget::LButtonDown(uint modKeys, TPoint& p)
{
	Window->IdleAction(0);

  TGadget::LButtonDown(modKeys, p);
  MenuItem->LButtonDown(modKeys, p, GetBounds());
}
//
void TMenuBarGadget::PaintBorder(TDC& dc)
{
	if(Window->GetFlatStyle()&TGadgetWindow::FlatStandard){
  	TBorderStyle oldStyle = BorderStyle;
  	if(GetEnabled() && IsHaveMouse())
    	BorderStyle = Raised;
  	TGadget::PaintBorder(dc);
    BorderStyle = oldStyle;
  }
  else
  	TGadget::PaintBorder(dc);
}
///////////////////////////////////////////////////////////////////////////////
//
class TFloatMenuBar : public TMenuBarProxy, public TDockableControlBar {
	public:
   	TFloatMenuBar(TWindow*        parent = 0,
                 	TTileDirection  direction = Horizontal,
                 	TFont*          font = new TGadgetWindowFont,
      				    TModule*        module = 0);
		~TFloatMenuBar(){}

		virtual TTileDirection GetDirection() const
    				    { return TDockableControlBar::GetDirection(); }
 		virtual TFont& GetFont() 
              	{ return TDockableControlBar::GetFont();      }
  	virtual TWindow* GetTarget()
    						{ return Target? Target : this;               }

  	void SetTarget(TWindow* target)
                { Target = target; }
		void SetMenu(HMENU hMenu);

	protected:
   	HMENU hMenu;
    TWindow* Target;
};
//
TFloatMenuBar::TFloatMenuBar(TWindow* parent, TTileDirection direction,
                 	     TFont* font, TModule* module)
:
	TDockableControlBar(parent,direction,font,module),
  hMenu(0),
  Target(0)
{
}
//
void TFloatMenuBar::SetMenu(HMENU hmenu)
{
	if(hMenu)
  	::DestroyMenu(hMenu);
  if(!hmenu)
  	return;
	hMenu = hmenu;

  TMenu menu(hMenu);
  int count = menu.GetMenuItemCount();
  _TCHAR buffer[MAX_PATH];
  for(int i = 0; i < count; i++){
    uint itemId 	 = menu.GetMenuItemID(i);
		uint itemState = menu.GetMenuState(i, MF_BYPOSITION);
    HMENU hMenu = menu.GetSubMenu(i);
    TMenuBarItem* item;
		menu.GetMenuString(i, buffer, MAX_PATH, MF_BYPOSITION);
    if(hMenu)
 			item = new TPopupMenuBarItem(this,itemId,hMenu,buffer);
    else
		  item = new TTextMenuBarItem(this,itemId, buffer);
	  Insert(*new TMenuBarGadget(item));
  }
}
////////////////////////////////////////////////////////////////////////////////
//
// Build a response table for all messages/commands handled
// by the application.
//
DEFINE_RESPONSE_TABLE1(TDockingApp, TApplication)
  EV_COMMAND(CM_HELPABOUT, CmHelpAbout),
  EV_COMMAND(CM_MDIFILENEW, CmFileNew),
  EV_COMMAND(CM_MDIFILEOPEN, CmFileOpen),
  EV_COMMAND(CM_FILEPRINT, CmFilePrint),
  EV_COMMAND(CM_FILEPRINTERSETUP, CmFilePrintSetup),
  EV_COMMAND(CM_FILEPRINTPREVIEW, CmFilePrintPreview),
  EV_COMMAND_ENABLE(CM_FILEPRINT, CePrint),
  EV_COMMAND_ENABLE(CM_FILEPRINTERSETUP, CePrint),
  EV_COMMAND_ENABLE(CM_FILEPRINTPREVIEW, CePrint),
END_RESPONSE_TABLE;


//
//
//
TDockingApp::TDockingApp() : TApplication("Docking Example")
{}


TDockingApp::~TDockingApp()
{}

static TDockable* 
newSpeedbar(TDecoratedFrame* frame, const char* caption)
{
  // Create command toolbar and associate toolbar buttons with commands.
  //
  TDockableControlBar* cb = new TDockableControlBar(frame);
  cb->SetCaption(caption);
  cb->Insert(*new TButtonGadget(CM_MDIFILENEW, CM_MDIFILENEW));
  cb->Insert(*new TButtonGadget(CM_MDIFILEOPEN, CM_MDIFILEOPEN));
  cb->Insert(*new TButtonGadget(CM_FILESAVE, CM_FILESAVE));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_EDITCUT, CM_EDITCUT));
  cb->Insert(*new TButtonGadget(CM_EDITCOPY, CM_EDITCOPY));
  cb->Insert(*new TButtonGadget(CM_EDITPASTE, CM_EDITPASTE));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_EDITUNDO, CM_EDITUNDO));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_EDITREDO, CM_EDITREDO));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_EDITFIND, CM_EDITFIND));
  cb->Insert(*new TButtonGadget(CM_EDITFINDNEXT, CM_EDITFINDNEXT));
  cb->Insert(*new TSeparatorGadget);
  cb->Insert(*new TButtonGadget(CM_FILEPRINT, CM_FILEPRINT));
  cb->Insert(*new TButtonGadget(CM_FILEPRINTPREVIEW, CM_FILEPRINTPREVIEW));

  // Add fly-over help hints.
  //
  cb->SetHintMode(TGadgetWindow::EnterHints);
  return cb;
}

void
TDockingApp::SetupSpeedBar(TDecoratedFrame* frame)
{
  TGadgetWindow::EnableFlatStyle();
  Harbor = new THarbor(*frame);

  // Create text style toolbar and associate toolbar buttons with commands.
  //
  TFloatMenuBar* cb2 = new TFloatMenuBar(frame);
  cb2->SetMenu(LoadMenu(IDM_MAIN));
  cb2->SetCaption("Menu Bar");

  // Insert some toolbars on the edges
  //
  Harbor->Insert(*cb2, alTop);
  Harbor->Insert(*newSpeedbar(frame, "ToolBar"), alTop);

}

//
// Application intialization.
//
void TDockingApp::InitMainWindow()
{
  if (nCmdShow != SW_HIDE)
    nCmdShow = (nCmdShow != SW_SHOWMINNOACTIVE) ? SW_SHOWNORMAL : nCmdShow;

  MdiClient = new TMDIClient;
  TDecoratedMDIFrame* frame = new TDecoratedMDIFrame(Name, 0, *MdiClient, true);

  // Assign ICON w/ this application.
  //
  frame->SetIcon(this, IDI_MDIAPPLICATION);

  // Associate with the accelerator table.
  //
  frame->Attr.AccelTable = IDA_MAIN;

  SetupSpeedBar(frame);

  TStatusBar *sb = new TStatusBar(frame, TGadget::Recessed,
                                  TStatusBar::CapsLock |
                                  TStatusBar::NumLock |
                                  TStatusBar::ScrollLock |
                                  TStatusBar::SizeGrip);

  // Show hints in the text gadget
  //
  sb->SetWideHints(false);
  frame->Insert(*sb, TDecoratedFrame::Bottom);

  SetMainWindow(frame);
  //frame->SetMenuDescr(TMenuDescr(IDM_MAIN, 1, 2, 0, 0, 1, 1));
}


//
// Menu Help About palette.exe command
//
void TDockingApp::CmHelpAbout()
{
  GetMainWindow()->MessageBox("Dockable toolbar Example", "About", MB_OK);
}

void TDockingApp::CmFileNew()
{
  char  title[255];

  // Generate a title for the MDI child window.
  //
  wsprintf(title, "%d", ChildCount++);

  TMDIChild* child = new TMDIChild(*MdiClient, title, new TEditFile(0, 0, 0));

  // Associate ICON w/ this child window.
  //
  child->SetIcon(this, IDI_DOC);

  // If the current active MDI child is maximize then this one should be also.
  //
  TMDIChild* curChild = MdiClient->GetActiveMDIChild();
  if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
    child->Attr.Style |= WS_MAXIMIZE;

  child->Create();
}

void TDockingApp::OpenFile(const char* fileName)
{
  if (fileName)
    strcpy(FileData.FileName, fileName);

  // Create a MDIChild window whose client is TEditFile.
  //
  TMDIChild* child = new TMDIChild(*MdiClient, "",
    new TEditFile(0, 0, 0, 0, 0, 0, 0, FileData.FileName));

  // Associate ICON w/ this child window.
  //
  child->SetIcon(this, IDI_DOC);

  // If the current active MDI child is maximize then this one should be also.
  //
  TMDIChild* curChild = MdiClient->GetActiveMDIChild();
  if (curChild && (curChild->GetWindowLong(GWL_STYLE) & WS_MAXIMIZE))
    child->Attr.Style |= WS_MAXIMIZE;

  child->Create();
}

//
// Menu File Open command
//
void TDockingApp::CmFileOpen()
{
  // Display standard Open dialog box to select a file name.
  //
  *FileData.FileName = 0;
  if (TFileOpenDialog(GetMainWindow(), FileData).Execute() == IDOK)
    OpenFile();
}

//
// Menu File Print command
//
void TDockingApp::CmFilePrint()
{
}

//
// Menu File Print Setup command
//
void TDockingApp::CmFilePrintSetup()
{
}

//
// Menu File Print Preview command
//
void TDockingApp::CmFilePrintPreview()
{
}

//
// Menu enabler used by Print, Print Setup and Print Preview.
//
void TDockingApp::CePrint(TCommandEnabler& ce)
{
  ce.Enable(true);
}

int OwlMain(int , char* [])
{
  return TDockingApp().Run();
}
